<?php
/*
Plugin Name: Homepage Pingbacks
Plugin URI: http://w-shadow.com/
Description: Adds a dashboard widget showing pingbacks to your homepage.
Version: 1.0
Author: Janis Elsts
Author URI: http://w-shadow.com/blog/
*/

class wsHomepagePingbacks {
    var $options;
    var $defaults;
    var $option_name = 'ws_homepage_pingbacks_settings';
    
    var $desired_db_version = 1;
    var $pingback_table;
    
    var $old_pingback_handler;
    
    function wsHomepagePingbacks(){
        global $wpdb;
        $this->pingback_table = $wpdb->prefix . 'homepage_pingbacks';
        
        $this->defaults = array(
            'db_version' => 0,
            'widget_items' => 10,  //Max. number of pingbacks to show in the widget.
            'show_domain' => true, //Show the domain name of the source URL besides the pingback title.
            'show_date' => false,  //Show when each pingback was received.
        );
        $this->load_options();
        
        if ( $this->options['db_version'] != $this->desired_db_version ){
            $this->install();
        }
        
        add_filter('xmlrpc_methods', array(&$this, 'override_pingback_handler'));
        add_action('wp_dashboard_setup', array(&$this, 'add_dashboard_widget'));
    }
    
    function override_pingback_handler($xmlrpc_methods){
        if ( array_key_exists('pingback.ping', $xmlrpc_methods) ){
            $this->old_pingback_handler = $xmlrpc_methods['pingback.ping'];
            $xmlrpc_methods['pingback.ping'] = array(&$this, 'pingback_ping');
        }
        return $xmlrpc_methods;
    }
    
    /**
     * Custom pingback handler that accepts pingbacks to the home page.
     * Pingbacks sent to other pages are passed on to the original handler.
     * 
     * @param array $args XMLRPC arguments
     * @return mixed
     */
    function pingback_ping($args){
        global $wpdb;
        
		$pagelinkedfrom = $args[0];
		$pagelinkedto   = $args[1];

		$title = '';

		$pagelinkedfrom = str_replace('&amp;', '&', $pagelinkedfrom);
		$pagelinkedto = str_replace('&amp;', '&', $pagelinkedto);
		$pagelinkedto = str_replace('&', '&amp;', $pagelinkedto);
        
        $homepage = get_option('home'); 

		//Check if the page linked to is the homepage
        $strip_strings = array('http://www.','http://','https://www.','https://', 'index.php');
        $strippped_pagelinkedto = trailingslashit(str_replace($strip_strings, '', $pagelinkedto));
        $strippped_homepage = trailingslashit(str_replace($strip_strings, '', $homepage));
        
        if ( $strippped_pagelinkedto != $strippped_homepage ){
            //No, it's some other page. Fall back to the original pingback handler.
            return $this->call_old_handler($args);
        }
        
        //Check that the remote site didn't already pingback the homepage
		if ( $wpdb->get_results( $wpdb->prepare("SELECT * FROM {$this->pingback_table} WHERE url = %s", $pagelinkedfrom) ) )
			return new IXR_Error( 48, __( 'The pingback has already been registered.' ) );
        
        /* 
        --- The following block of code was copied verbatim from wp_xmlrpc_server::pingback_ping() --- 
        It verifies the presence of our link on the linked-from page and extracts its title + context.  
        */
        
		// Let's check the remote site
		$linea = wp_remote_fopen( $pagelinkedfrom );
		if ( !$linea )
	  		return new IXR_Error(16, __('The source URL does not exist.'));

		$linea = apply_filters('pre_remote_source', $linea, $pagelinkedto);

		// Work around bug in strip_tags():
		$linea = str_replace('<!DOC', '<DOC', $linea);
		$linea = preg_replace( '/[\s\r\n\t]+/', ' ', $linea ); // normalize spaces
		$linea = preg_replace( "/ <(h1|h2|h3|h4|h5|h6|p|th|td|li|dt|dd|pre|caption|input|textarea|button|body)[^>]*>/", "\n\n", $linea );

		preg_match('|<title>([^<]*?)</title>|is', $linea, $matchtitle);
		$title = $matchtitle[1];
		if ( empty( $title ) )
			return new IXR_Error(32, __('We cannot find a title on that page.'));

		$linea = strip_tags( $linea, '<a>' ); // just keep the tag we need

		$p = explode( "\n\n", $linea );

		$preg_target = preg_quote($pagelinkedto, '|');

		foreach ( $p as $para ) {
			if ( strpos($para, $pagelinkedto) !== false ) { // it exists, but is it a link?
				preg_match("|<a[^>]+?".$preg_target."[^>]*>([^>]+?)</a>|", $para, $context);

				// If the URL isn't in a link context, keep looking
				if ( empty($context) )
					continue;

				// We're going to use this fake tag to mark the context in a bit
				// the marker is needed in case the link text appears more than once in the paragraph
				$excerpt = preg_replace('|\</?wpcontext\>|', '', $para);

				// prevent really long link text
				if ( strlen($context[1]) > 100 )
					$context[1] = substr($context[1], 0, 100) . '...';

				$marker = '<wpcontext>'.$context[1].'</wpcontext>';    // set up our marker
				$excerpt= str_replace($context[0], $marker, $excerpt); // swap out the link for our marker
				$excerpt = strip_tags($excerpt, '<wpcontext>');        // strip all tags but our context marker
				$excerpt = trim($excerpt);
				$preg_marker = preg_quote($marker, '|');
				$excerpt = preg_replace("|.*?\s(.{0,100}$preg_marker.{0,100})\s.*|s", '$1', $excerpt);
				$excerpt = strip_tags($excerpt); // YES, again, to remove the marker wrapper
				break;
			}
		}

		if ( empty($context) ) // Link to target not found
			return new IXR_Error(17, __('The source URL does not contain a link to the target URL, and so cannot be used as a source.'));

		$pagelinkedfrom = str_replace('&', '&amp;', $pagelinkedfrom);

		$context = '[...] ' . esc_html( $excerpt ) . ' [...]';
        
        /* --- End of copied code block --- */
        
        $this->save_pingback($pagelinkedfrom, $title, $context);
        
		return sprintf(__('Pingback from %1$s to %2$s registered. Keep the web talking! :-)'), $pagelinkedfrom, $pagelinkedto);
    }
    
    /**
     * Store a homepage pingback in the DB.
     * Note : Doesn't check if the pingback was already registered. 
     * 
     * @param string $url
     * @param string $title
     * @param string $context
     * @return void
     */
    function save_pingback($url, $title, $context){
        global $wpdb;
        $values = compact('url', 'title', 'context');
        $values['date'] = current_time('mysql');
        $wpdb->insert($this->pingback_table, $values);                       
    }
    
    /**
     * Call the original pingback handler. 
     * 
     * This is basically a slightly modified version of IXR_Server::call(). It should be able
     * to handle multiple plugins overriding 'pingback.ping'.
     * 
     * Note that we assume that the default WP XMLRPC server is available in the global
     * $wp_xmlrpc_server variable. 
     * 
     * @param mixed $args
     * @return mixed
     */
    function call_old_handler($args) {
        global $wp_xmlrpc_server;
        $method = $this->old_pingback_handler;
        
        //Some input validation is skipped here because the original call() method
        //will have already performed it.

        // Are we dealing with a function or a method?
        if (is_string($method) && substr($method, 0, 5) == 'this:') {
            // It's a class method - presumably from $wp_xmlrpc_server. Check if it exists.
            $method = substr($method, 5);
            if (!method_exists($wp_xmlrpc_server, $method)) {
                return new IXR_Error(-32601, 'server error. requested class method "'.$method.'" does not exist.');
            }

            //Call the method
            $result = $wp_xmlrpc_server->$method($args);
        } else {
            // It's a function - does it exist?
            if (is_array($method)) {
                if (!method_exists($method[0], $method[1])) {
                    return new IXR_Error(-32601, 'server error. requested object method "'.$method[1].'" does not exist.');
                }
            } else if (!function_exists($method)) {
                return new IXR_Error(-32601, 'server error. requested function "'.$method.'" does not exist.');
            }

            // Call the function
            $result = call_user_func($method, $args);
        }
        return $result;
    }
    
    /**
     * Register the dashboard wiget. 
     * 
     * @return void
     */
    function add_dashboard_widget(){
        if ( function_exists( 'wp_add_dashboard_widget' ) ) {
			wp_add_dashboard_widget(
				'ws_homepage_pingbacks_widget', 
				'Incoming Homepage Pingbacks', 
				array( &$this, 'dashboard_widget' ),
				array( &$this, 'dashboard_widget_control' )
			 );
		} 
    }
    
    /**
     * Generate and output the contents of the dashboard widget.
     * 
     * @return void
     */
    function dashboard_widget(){
        $pingbacks = $this->fetch_recent_pingbacks($this->options['widget_items']);
        
        if ( empty($pingbacks) ){
            echo 'No pingbacks yet.';
            return;
        }
        
        echo '<div class="rss-widget"><ul>';
        
        foreach ( $pingbacks as $pingback ) {
    		$link = esc_url( strip_tags( $pingback['url'] ) );
            $title = esc_html( strip_tags( $pingback['title'] ) );
            $context = esc_html( strip_tags($pingback['context']) );
            $date = strtotime($pingback['date']);
            
            echo '<li>';
            
            printf('<a class="rsswidget" href="%s" title="%s">%s</a>', $link, $context, $title);
            
            if ( $this->options['show_domain'] ){
                $parsed = @parse_url($pingback['url']);
                if ( !empty($parsed['host']) ){
                    printf(
                        ' <span class="nonessential pingback-domain">(%s)</span>', 
                        esc_html( strip_tags($parsed['host']) )
                    );
                }
            }
            
            if ( $this->options['show_date'] ){
                printf(' <span class="nonessential">%s</span>', date_i18n('F j, Y', $date));
            }
            
            echo '</li>';
    	}
        echo '</ul></div>';
    }
    
    /**
     * Display and process the widget's configuration form.
     * 
     * @param mixed $arg1 Unknown.
     * @param mixed $arg2 Unknown.
     * @return void
     */
    function dashboard_widget_control( $arg1 = null, $arg2 = null ){
		if ( 'POST' == $_SERVER['REQUEST_METHOD'] && 'ws_homepage_pingbacks_widget' == $_POST['widget_id'] ) {
			$this->options['widget_items'] = intval($_POST['ws_hp_widget_items']);
            $this->options['show_date'] = !empty($_POST['ws_hp_show_date']);
            $this->options['show_domain'] = !empty($_POST['ws_hp_show_domain']);
			$this->save_options();
		}
		?>
		<p><label>
            How many items would you like to display?
            <select name="ws_hp_widget_items">
                <?php
                for( $i = 1; $i <= 20; $i++ ){
                    printf(
                        '<option value="%1$d"%2$s>%1$d </option>',
                        $i, 
                        $this->options['widget_items'] == $i ? ' selected="selected"' : ''
                    );
                }
                ?>
            </select> 
		</label></p>
        
        <p><label>
            <input type="checkbox" name="ws_hp_show_date" <?php if( $this->options['show_date'] ) echo 'checked="checked"'; ?> />
            Display item date
        </label></p>
        
        <p><label>
            <input type="checkbox" name="ws_hp_show_domain" <?php if( $this->options['show_domain'] ) echo 'checked="checked"'; ?> />
            Display domain name
        </label></p>
        
        </p>
		<?php
    }
    
    /**
     * Retrieve the latest homepage pingbacks.
     * 
     * @param integer $limit Max. items to return.
     * @return array 
     */
    function fetch_recent_pingbacks($limit = 10){
        global $wpdb;
        $limit = intval($limit);
        
        $q = "SELECT `url`, `title`, `context`, `date` FROM {$this->pingback_table} ORDER BY `date` DESC";
        if ( $limit ){
            $q .= " LIMIT $limit";
        }
        
        return $wpdb->get_results($q, ARRAY_A);
    }
    
    function load_options(){
        $this->options = $this->defaults;
        $options = get_option($this->option_name);
        if ( is_array($options) ){
            $this->options = array_merge($this->options, $options);
        }
    }
    
    function save_options(){
        if ( is_array($this->options) ){
            update_option($this->option_name, $this->options);
        }
    }
    
    /**
     * Create or update the plugin's DB tables.
     * 
     * @return void
     */
    function install(){
        if ( !function_exists('dbDelta') ){
            require ABSPATH . '/wp-admin/includes/upgrade.php';
        }
        
        $q = "CREATE TABLE `{$this->pingback_table}` (
                `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `url` varchar(255) NOT NULL,
                `title` tinytext NOT NULL,
                `context` text NOT NULL,
                `date` datetime NOT NULL,
                PRIMARY KEY (`id`)
            )";
        dbDelta($q);
        
        $this->options['db_version'] = $this->desired_db_version;
        $this->save_options();
    }
}

new wsHomepagePingbacks(); //This works because a ref. to the plugin object is stored in WP's hook arrays.

?>